package PureMessage::V6::Migrate::Quarantine;
use strict;
use warnings;
use base 'PureMessage::V6::Migrate::Base';
use PureMessage::V6::Migrate::Quarantine::Dir;
use PureMessage::V6::Migrate::Quarantine::DB;
use PureMessage::V6::Migrate::Utils qw(get_free_space);
use PureMessage::Version qw($PREFIX);
use ActiveState::Prompt qw(yes);
use Term::ANSIColor qw(color);
use ActiveState::DiskUsage qw(du);

our $SCRIPT_DIR = "$PREFIX/var/migrate/v6";

my @WORKER_MODULES = qw(DB Dir);

sub new {
    my $class = shift;
    my $self = $class->SUPER::new(@_);

    for my $p (@WORKER_MODULES) {
        my $mod = __PACKAGE__ . "::$p";
        $self->{$p} = $mod->new(@_, no_exit => 1);
    }
    return $self;
}

sub display {
    my $self = shift;

    for my $p (@WORKER_MODULES) {
        return 0 unless $self->{$p}->display();
    }

    $self->show(<<EOT);

This option will migrate your database and quarantine from PureMessage version
5 to version 6. The Quarantine and database data files will be deleted and
replaced with their counterparts from version 5. 

Depending on the amount of data you are migrating, this step may take several
hours to finish and if it should fail, it can be re-run once the cause of the
failure is addressed.
EOT

    my $msg = color 'bold';
    $msg .= <<EOT;
Important: The migration requires some slight changes to your PureMessage v5
installation that will be reverted when this step is done. More information is
provided in the PureMessage 6 upgrade guide
EOT
    $msg .= color 'reset';

    $self->show($msg);

    if (yes("Would you like to proceed (yes/no)?", 0)) {
        return 1;
    }

    return 0;
}

sub migrate {
    my $self = shift;

    $self->log_info1("Checking required diskspace...");

    my $qsize = $self->check_disk_space();
    my $total = int ($qsize->{TOTAL} * 1.2);
    $self->log2file('INFO', "Diskspace: required: $total, free: " .
                    $qsize->{FREE} . ", overhead: $qsize->{OVERHEAD}");

    if ($qsize->{FREE} + $qsize->{OVERHEAD} < $total) {
        $self->log_err("Insufficient diskspace, required=$total, " .
                       "free=$qsize->{FREE}, overhead=$qsize->{OVERHEAD}");
        return 0;
    }

    $self->log_info1("Diskspace ok");

    for my $p (@WORKER_MODULES) {
        my $rc = $self->{$p}->migrate(@_, QSIZE => $qsize);
        return $rc unless $rc;
    }
    return 1;
}

sub check_disk_space {
    my $self = shift;
    my %qsize;

    $qsize{FREE} = get_free_space("$PREFIX/.");
    $self->pipeline_as_pmxv5(
        "$SCRIPT_DIR/pmx-v5qdiskspace --has-db $self->{has_db}",
        sub {
            chomp(my $line = shift || '');
            my $data = (split /\t/, $line)[-1];
            my ($k, $v) = split /\s*=\s*/, $data, 2;
            $qsize{uc $k} = $v;
        },
        filter => qr/QSIZE:\t/,
    );
    $qsize{OVERHEAD} = $self->get_current_usage();
    return \%qsize;
}

sub get_current_usage {
    my $self = shift;

    my %dirs = (
       DB  => "$PREFIX/postgres/var/data",
       DIR => "$PREFIX/var/qdir",
    );
    my $total = 0;

    for my $dir (keys %dirs) {
        next if (!$self->{has_db} and ($dir eq 'DB'));

        my $path = $dirs{$dir};
        my $used = du("$path/.");
        $total += $used;
    }
    return $total;
}

1;
