package PureMessage::V6::Migrate::Quarantine::DB;
use strict;
use warnings;
use base 'PureMessage::V6::Migrate::Base';

use PureMessage::Config qw(%config);
use ActiveState::Config;
use File::Copy;
use PureMessage::Version qw($PREFIX);
use PureMessage::V6::Migrate::Utils qw(
    chown_as_pmxv5 chown_as_pmxv6 
    get_pmxv6_user get_pmxv5_user
);

our $SCRIPT_DIR = "%%PREFIX6%%/var/migrate/v6";
my $PATCHED_SCHEMA_VER = '5.601';

sub display {
    my $self = shift;

    if (! $self->{has_db}) {
        $self->log2file("INFO", "No Database detected.");
        return 1;
    }

    return 1;
}

sub migrate {
    my $self = shift;

    if (! $self->migrate_db()) {
        return 0;
    }

    my $db_location;
    if ($self->{has_db}) {
        $db_location = 'local';
        $self->log_info1("Resetting location id's for all edges");
        $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6location reset");
    }
    else {
        $db_location = 'remote';

        my $id;
        $self->log_info1("Reclaiming location id");
        $self->pipeline_as_pmxv5("$SCRIPT_DIR/pmx-v5locid",
            sub {
                chomp(my $line = shift || '');
                $id = (split /\t/, $line)[-1];
            },
            filter => qr/^LOCID:\t/,
        );

        
        $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6location reclaim $id");
    }

    # Sync to DB quarantine-expire resource before we sync back from db. 
    # we do this in order to prevent archive-dir be overwritten after
    # migration
    $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6resync to-db quarantine-expire");

    # Now synchronize the DB contents to disk (resources)
    $self->log_info1("Synchronizing local files from ($db_location) DB");
    $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6resync from-db");

    return 1;
}

sub migrate_db {
    my $self = shift;

    if (! $self->{has_db}) {
        $self->log2file("INFO", "No Database detected.");
        return 1;
    }
   
    # stop all pmxv5/6 services
    $self->log_info1("Stopping all PureMessage v5/v6 services");
    $self->stop_all_services();

    # trying to recover pmxv5 and pmxv6 in case they are broken
    $self->recover_pmx();

    $self->run_as_pmxv5("pmx-database start");
    $self->load_dbinfo();
    if (!$self->check_requirements()) {
        $self->run_as_pmxv5("pmx-database stop");
        return 0;
    }
    $self->log_info1("Starting database migration");

    my $dbinfo = $self->{DBINFO};
    if ($self->need_to_patch_db($dbinfo->{SCHEMA_VERSION})) {
        $self->log2file("INFO", "Patching Database...");
        # patch v5 db & prep
        $self->run_as_pmxv5("$SCRIPT_DIR/pmx-v5db-fix --action prepare");
    } else {
        $self->log2file("INFO", "Skip patching Database...");
    }
    $self->run_as_pmxv5("pmx-database stop");

    $self->pre_migration();

    # run pg_upgrade
    $self->log_info2("Migrating Database (This may take several hours to run)");
    my $ping = sub {
        my $line = shift;

        $self->ping;
        $self->log2file("INFO", $line);
    };
    eval {
        $self->pipeline_as_pmxv5(
            "LD_LIBRARY_PATH=%%PREFIX6%%/lib %%PREFIX6%%/postgres/bin/pg_upgrade " .
            "-p 5434 " .
            "-P 5433 " .
            "-b %%PREFIX5%%/postgres/bin -B %%PREFIX6%%/postgres/bin " .
            "-d %%PREFIX5%%/postgres/var/data/ -D %%PREFIX6%%/postgres/var/data",
            $ping,
            raw_cmd => 1,
        );
    };
    if ($@) {
        # if pg_upgrade fails, trying to recover pmxv5/6
        $self->recover_pmx();
        return 0;
    }

    $self->post_migration();

    eval {
        $self->run_as_pmxv6("pmx-database start",  exitcodes => [2]);
        $self->pipeline_as_pmxv6(
            "%%PREFIX6%%/postgres/bin/vacuumdb -d pmx_quarantine -Z -v", 
            $ping,
            raw_cmd => 1,
        );
        $self->run_as_pmxv6("pmx-database stop",  exitcodes => [2]);
    };
    if ($@) {
        # if vacuumdb fails, trying to recover pmxv5/6
        $self->recover_pmx();
        return 0;
    }

    # test v6 db
    if ($self->test_db()) {
        $self->log_info2("Database migration complete");
        return 1;
    } else {
        $self->log_info1("Unable to restart PureMessage v6 database");
        return 0;
    }
}

sub check_requirements {
    my $self = shift;
    my $dbinfo = $self->{DBINFO};

    my $lc_collate = $dbinfo->{LC_COLLATE};
    if ($lc_collate) {
        if (uc($lc_collate) ne 'C') {
            $self->log_info1(" ");
            $self->log_info1("Unable to migrate database data due to locale mismatch (current: $lc_collate, required: C).");
            $self->log_info1("Instructions for migrating the database data manually can be found here:");
            $self->log_info1(" ");
            $self->log_info2("http://www.sophos.com/support/knowledgebase/article/113815.html");
            $self->log_info1(" ");
            return 0;
        }
    } else {
        die "Cannot detect lc_collate\n";
    }
    return 1;
}

sub need_to_patch_db {
    my ($self, $cur_schema_ver) = @_;

    my $cur_ver = $cur_schema_ver;
    my $patched_ver = $PATCHED_SCHEMA_VER;

    $cur_ver =~ s/\.//g;
    $patched_ver =~ s/\.//g;

    if ($cur_ver < $patched_ver) {
        return 1;
    }
    return 0;
}

sub load_dbinfo {
    my $self = shift;
    my %dbinfo;
    $self->pipeline_as_pmxv5(
        "$SCRIPT_DIR/pmx-v5dbinfo",
        sub {
            chomp (my $line = shift || '');
            my $data = (split /\t/, $line)[-1];
            my ($k, $v) = split /\s*=\s*/, $data, 2;
            $dbinfo{uc $k} = $v;
        },
        filter => qr/DBINFO:\t/,
    );
    $self->{DBINFO} = \%dbinfo;
}

sub post_migration {
    my ($self) = @_;

    my $pmxv5_user = get_pmxv5_user();
    my $pmxv6_user = get_pmxv6_user();

    # restore v5 db state
    $self->run_as_pmxv5("$SCRIPT_DIR/pmx-v5db-fix --action recover");

    # restore ownership of v6
    chown_as_pmxv6("%%PREFIX6%%/postgres");

    # rename superuser
    $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6rename-role --from $pmxv5_user --to $pmxv6_user");
    
    # apply all db patch
    $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6postdb");

    # restore pg conf files
    $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6db --recover-conf");
}

sub pre_migration {
    my ($self) = @_;

    # rebuild v6 db (clean state)
    $self->log_info2("Rebuilding PureMessage v6 Database");
    my $pmxv5_user = get_pmxv5_user();
    $self->run_as_pmxv6("$SCRIPT_DIR/pmx-v6db --rebuild --pg-user $pmxv5_user");

    # change onwership of v6 db
    chown_as_pmxv5("%%PREFIX6%%/postgres");
}

sub recover_pmx {
    my ($self) = @_;

    # restore v5 db state
    $self->run_as_pmxv5("$SCRIPT_DIR/pmx-v5db-fix --action recover");

    # trying to fix bad db state
    $self->stop_leftover_db();
    chown_as_pmxv6("%%PREFIX6%%/postgres");
}

sub test_db {
    my ($self) = @_;

    eval {
        $self->run_as_pmxv6("pmx-database start");
    };
    if ($@) {
        return 0;
    }

    return 1;
}

sub stop_leftover_db {
    my ($self) = @_;

    $self->run_as_root(
        "%%PREFIX6%%/bin/pmx-env " . 
        "$SCRIPT_DIR/pmx-v5stop-db " .
        "--prefix-v5=%%PREFIX5%% " . 
        "--prefix-v6=%%PREFIX6%%");
}

sub stop_all_services {
    my ($self) = @_;

    if (-e "$PREFIX/sendmail" || -e "$PREFIX/postfix") {
        $self->run_as_pmxv6("pmx-service stop smtp");
    }
    $self->run_as_pmxv6("pmx stop");
}

1;
